from ts3audiobot_api.ts3commandcaller import CommandCaller
import base64
import requests


def generate_base64(string):
    message_bytes = string.encode('ascii')
    base64_bytes = base64.b64encode(message_bytes)
    base64_message = base64_bytes.decode('ascii')
    return base64_message


class TS3AudioBot:
    """
    creates the node connection
    """

    def __init__(self, ip, api_token, port=58913, timeout=5):
        """
        Initialization of Node Class
        :param api_token: string
        :param ip:
        :param port:
        :param timeout:
        """

        self.ip = ip
        self.port = port
        self.timeout = timeout
        self.api_token = api_token
        self.username = api_token.split(":")[0]
        self.access_token = api_token.split(":")[1]
        self.commandExecutor = CommandCaller(self)
        self.bot_id = 0

        self.base_raw = "http://{ip}:{port}/api/{endpoint}"
        self.base = "http://{ip}:{port}/api/bot/use/{bot_id}/(/{endpoint}"

    def generate_header(self):
        return "Basic {token}".format(token=generate_base64(self.api_token))

    def request(self, endpoint):

        r = requests.get(self.base.format(ip=self.ip, port=self.port, bot_id=self.bot_id, endpoint=endpoint), headers={
            "Authorization": self.generate_header(),
        }, timeout=self.timeout)

        try:
            return r.json()
        except ValueError:
            return True

    def raw_request(self, endpoint):

        r = requests.get(self.base_raw.format(ip=self.ip, port=self.port, endpoint=endpoint), headers={
            "Authorization": self.generate_header(),
        }, timeout=self.timeout)

        try:
            return r.json()
        except ValueError:
            return True

    def get_command_executor(self):
        return self.commandExecutor

    def get_port(self):
        return self.port

    def set_port(self, port):
        self.port = port

    def get_ip(self):
        return self.ip

    def set_ip(self, ip):
        self.ip = ip

    def get_current_id(self):
        return self.bot_id




from urllib.parse import urlparse


def is_url(link):
    """
    Check if link is valid
    """
    try:
        result = urlparse(link)
        if all([result.scheme, result.netloc]):
            return True
    except ValueError:
        return False


class Playlist:

    def __init__(self, ts3audiobot):
        self.ts3audiobot = ts3audiobot

    def add(self, link):
        if is_url(link):
            return self.ts3audiobot.request("list/add/{}".format(link))
        else:
            return False

    def clear(self):
        return self.ts3audiobot.request("list/clear")

    def delete(self):
        return self.ts3audiobot.request("delete")

    def get(self):
        return self.ts3audiobot.request("list/get")

    def item_move(self):
        return self.ts3audiobot.request("list/item/move")

    def item_delete(self):
        return self.ts3audiobot.request("list/item/delete")

    def list(self):
        return self.ts3audiobot.request("list/list")

    def load(self):
        return self.ts3audiobot.request("list/load")

    def merge(self):
        return self.ts3audiobot.request("list/merge")

    def rename(self):
        return self.ts3audiobot.request("list/name")

    def play(self):
        return self.ts3audiobot.request("list/play")

    def play_from(self, index):
        return self.ts3audiobot.request("list/play/{}".format(index))

    def get_queue(self):
        return self.ts3audiobot.request("list/queue")

    def save(self):
        return self.ts3audiobot.request("list/save")

    def show(self):
        return self.ts3audiobot.request("list/show")

    def random(self):
        return self.ts3audiobot.request("random")

    def enable_random(self):
        return self.ts3audiobot.request("random/on")

    def disable_random(self):
        return self.ts3audiobot.request("random/off")


class History:

    def __int__(self, ts3audiobot):
        self.ts3audiobot = ts3audiobot

    def add(self, id):
        return self.ts3audiobot.request("history/add/{id}".format(id=id))

    def clean(self, remove_defective=None):

        if not remove_defective:
            return self.ts3audiobot.request("history/clean")
        else:
            return self.ts3audiobot.request("history/clean/remove_defective")

    def delete(self, id):
        return self.ts3audiobot.request("history/delete/{id}".format(id=id))

    def history_form(self, count, userBDid):
        return self.ts3audiobot.request("history/from/{}/{}".format(count, userBDid))

    def history_id(self, id):
        return self.ts3audiobot.request("history/id/{}".format(id))

    def last(self, count):
        return self.ts3audiobot.request("history/last/{}".format(count))

    def play_last(self, count):
        return self.ts3audiobot.request("history/last/{}".format(count))

    def play(self, id):
        return self.ts3audiobot.request("history/play/{}".format(id))

    def rename(self, id, name):
        return self.ts3audiobot.request("history/rename/{}/{}".format(id, name))

    def till(self, date):
        return self.ts3audiobot.request("history/till/{}".format(date))

    def filter_titel(self, title):
        return self.ts3audiobot.request("history/title/{}".format(title))


class User:

    def __init(self, ts3audiobot):
        self.ts3audiobot = ts3audiobot

    def get_user_uid_by_id(self):
        return self.ts3audiobot.request("getuser/uid/byid")

    def get_username_by_id(self):
        return self.ts3audiobot.request("getuser/name/byid")

    def get_user_dbid_by_id(self):
        return self.ts3audiobot.request("getuser/dbid/byid")

    def get_user_channel_by_id(self):
        return self.ts3audiobot.request("getuser/channel/byid")

    def get_user_all_by_id(self):
        return self.ts3audiobot.request("getuser/all/byid")

    def get_user_id_by_name(self):
        return self.ts3audiobot.request("getuser/id/byname")

    def get_username_by_dbid(self):
        return self.ts3audiobot.request("getuser/name/bydbid")
   
